@@ -1,6 +1,6 @@
 package Net::DNS;
 #
-# $Id: DNS.pm 388 2005-06-22 10:06:05Z olaf $
+# $Id: DNS.pm 480 2005-07-31 14:23:50Z olaf $
 #
 use strict;
 
@@ -41,7 +41,7 @@ BEGIN {
     @ISA     = qw(Exporter DynaLoader);
 
     
-    $VERSION = '0.51_02';
+    $VERSION = '0.53_01';
     $HAVE_XS = eval { 
 	local $SIG{'__DIE__'} = 'DEFAULT';
 	__PACKAGE__->bootstrap(); 1 
@@ -181,10 +181,9 @@ sub typesbyval {
     my $val = shift;
     
     die "Net::DNS::typesbyval() argument ($val) is not numeric" unless 
-        $val =~ m/^\s*\d+\s*$/;  
+	$val =~ m/^\s*0*(\d+)\s*$/;
+    $val = $1;
     
-    $val =~ s/\s*//g; 
-    $val =~ s/^0*([0-9]+)/$1/;    #remove leading zeros 
     
     return $typesbyval{$val} if $typesbyval{$val};
     
