@@ -1,16 +1,21 @@
 package Net::DNS::RR;
 #
-# $Id: RR.pm 264 2005-04-06 09:16:15Z olaf $
+# $Id: RR.pm 388 2005-06-22 10:06:05Z olaf $
 #
 use strict;
-use vars qw($VERSION $AUTOLOAD);
 
+BEGIN { 
+    eval { require bytes; }
+} 
+
+use vars qw($VERSION $AUTOLOAD);
 use Carp;
 use Net::DNS;
 use Net::DNS::RR::Unknown;
 
 
-$VERSION = (qw$LastChangedRevision: 264 $)[1];
+
+$VERSION = (qw$LastChangedRevision: 388 $)[1];
 
 =head1 NAME
 
@@ -608,6 +613,7 @@ sub data {
 		$data  = $packet->dn_comp($self->{'name'}, $offset);
 	}
 
+
 	my $qtype     = uc($self->{'type'});
 	my $qtype_val = ($qtype =~ m/^\d+$/) ? $qtype : Net::DNS::typesbyname($qtype);
 	$qtype_val    = 0 if !defined($qtype_val);
@@ -631,8 +637,9 @@ sub data {
 
 	my $rdata = $self->rdata($packet, $offset);
 
+
 	$data .= pack('n', length $rdata);
-	$data .= $rdata;
+	$data.=$rdata;
 
 	return $data;
 }
