@@ -1,6 +1,6 @@
 package Net::DNS::RR::OPT;
 #
-# $Id: OPT.pm 388 2005-06-22 10:06:05Z olaf $
+# $Id: OPT.pm 393 2005-07-01 19:21:52Z olaf $
 #
 
 use strict;
@@ -12,7 +12,7 @@ use vars qw(@ISA $VERSION %extendedrcodesbyname %extendedrcodesbyval $EDNSVERSIO
 use Carp;
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 388 $)[1];
+$VERSION = (qw$LastChangedRevision: 393 $)[1];
 
 $EDNSVERSION = 0;
 
@@ -64,6 +64,7 @@ sub new {
 
 
 
+
 sub new_from_string {
     my ($class, $self ) = @_;
     
@@ -107,6 +108,7 @@ sub new_from_hash {
 
 
 
+
 sub string {
    my $self = shift;
    return
@@ -140,6 +142,44 @@ sub rr_rdata {
 }
 
 
+
+
+
+
+
+sub do{
+    my $self=shift;
+    return ( 0x8000 & $self->{"ednsflags"} );
+}
+
+
+
+sub set_do{
+    my $self=shift;
+    return $self->{"ednsflags"} = ( 0x8000 | $self->{"ednsflags"} );
+
+}
+
+
+
+sub clear_do{
+    my $self=shift;
+    return $self->{"ednsflags"} = ( ~0x8000 & $self->{"ednsflags"} );
+
+}
+
+
+
+sub size {
+    my $self=shift;
+    my $size=shift;
+    $self->{"class"}=$size if defined($size);
+    return $self->{"class"};
+}
+
+
+
+
 1;
 __END__
 
@@ -188,6 +228,21 @@ If optioncode is left undefined then we do not expect any RDATA.
 The defaults are no rdata.   
 
 
+=head2 do, set_do, clear_do
+
+    $opt->set_do;
+
+Reads, sets and clears the do flag. (first bit in the ednssflags);
+
+
+=head2 size
+
+    $opt->size(1498);
+    print "Packet size:". $opt->size() ;
+ 
+Sets or gets the packet size.
+
+
 =head1 TODO
 
 - This class is tailored to use with dnssec. 
