@@ -1,6 +1,6 @@
 package Net::DNS::Resolver::Base;
 #
-# $Id: Base.pm 388 2005-06-22 10:06:05Z olaf $
+# $Id: Base.pm 479 2005-07-31 14:19:41Z olaf $
 #
 
 use strict;
@@ -25,7 +25,7 @@ use Net::IP qw(ip_is_ipv4 ip_is_ipv6 ip_normalize);
 use Net::DNS;
 use Net::DNS::Packet;
 
-$VERSION = (qw$LastChangedRevision: 388 $)[1];
+$VERSION = (qw$LastChangedRevision: 479 $)[1];
 
 
 #
@@ -574,8 +574,8 @@ sub send_tcp {
 		my $sock;
 		my $sock_key = "$ns:$dstport";
 		my ($host,$port);
-		if ($self->persistent_tcp && $self->{'sockets'}{$sock_key}) {
-			$sock = $self->{'sockets'}{$sock_key};
+		if ($self->persistent_tcp && $self->{'sockets'}[AF_UNSPEC]{$sock_key}) {
+			$sock = $self->{'sockets'}[AF_UNSPEC]{$sock_key};
 			print ";; using persistent socket\n"
 				if $self->{'debug'};
 		} else {
@@ -587,13 +587,17 @@ sub send_tcp {
 		    #my $old_wflag = $^W;
 		    #$^W = 0;
 		    if ($has_inet6 && ! $self->force_v4()){
-			$srcaddr="0" if $srcaddr eq "0.0.0.0";  # Otherwise the INET6 socket will just fail
-			
+                        # XXX IO::Socket::INET6 fails in a cryptic way upon send()
+                        # on AIX5L if "0" is passed in as LocalAddr
+			# $srcaddr="0" if $srcaddr eq "0.0.0.0";  # Otherwise the INET6 socket will just fail
+
+                        my $srcaddr6 = $srcaddr eq '0.0.0.0' ? '::' : $srcaddr;
+
 			$sock = 
 			    IO::Socket::INET6->new(
 						   PeerPort =>    53,
 						   PeerAddr =>    $ns,
-						   LocalAddr => 0,
+						   LocalAddr => $srcaddr6,
 						   LocalPort => ($srcport || undef),
 						   Proto     => 'tcp',
 						   Timeout   => $timeout,
@@ -619,7 +623,7 @@ sub send_tcp {
 			next;
 		    }
 
-			$self->{'sockets'}{$sock_key} = $sock;
+			$self->{'sockets'}[AF_UNSPEC]{$sock_key} = $sock;
 		}
 
 		my $lenmsg = pack('n', length($packet_data));
@@ -742,9 +746,11 @@ sub send_udp {
 	
 	if ($has_inet6  && ! $self->force_v4() && !defined( $sock[AF_INET6()] )){
 
-	    
-	    $srcaddr="0" if $srcaddr eq "0.0.0.0";  # Otherwise the INET6 socket will just fail
 
+	    # '::' Otherwise the INET6 socket will fail.
+	    
+            my $srcaddr6 = $srcaddr eq '0.0.0.0' ? '::' : $srcaddr;
+	    
 	    print ";; Trying to set up a AF_INET6() family type UDP socket with srcaddr: $srcaddr ... "
 		if $self->{'debug'};
 
@@ -756,8 +762,8 @@ sub send_udp {
 	    #my $old_wflag = $^W;
 	    #$^W = 0;
 	    
-	    $sock[AF_INET6] = IO::Socket::INET6->new(
-						       LocalAddr => $srcaddr,
+	    $sock[AF_INET6()] = IO::Socket::INET6->new(
+						       LocalAddr => $srcaddr6,
 						       LocalPort => ($srcport || undef),
 						       Proto     => 'udp',
 						       );
@@ -1255,8 +1261,8 @@ sub axfr_start {
 	my $sock;
 	my $sock_key = "$ns:$self->{'port'}";
 
-	if ($self->{'persistent_tcp'} && $self->{'sockets'}->{$sock_key}) {
-	    $sock = $self->{'sockets'}->{$sock_key};
+	if ($self->{'persistent_tcp'} && $self->{'sockets'}[AF_UNSPEC]{$sock_key}) {
+	    $sock = $self->{'sockets'}[AF_UNSPEC]{$sock_key};
 	    print ";; using persistent socket\n" if $self->{'debug'};
 	    
 	} else {
@@ -1288,10 +1294,10 @@ sub axfr_start {
 		$self->errorstring('connection failed');
 		print ';; ERROR: send_tcp: connection ',
 		"failed: $!\n" if $self->{'debug'};
-		next;
+		return;
 	    }
 	    
-	    $self->{'sockets'}{$sock_key} = $sock;
+	    $self->{'sockets'}[AF_UNSPEC]{$sock_key} = $sock;
 	}
 
 
